﻿unit DelphiCli.Templates;

interface

uses
  System.Classes, System.Generics.Collections, System.SysUtils,
  DelphiCli.Options, DelphiCli.Entities;

type

  TTemplateManager = class
  private
    class var FTemplateNames: TDictionary<TProjectType, string>;
    class procedure ExtractTemplateToDirectory(const ATemplatName: string;
      const ADestPath: string);
    class procedure PatchFileContents(const AFilePaths: TArray<string>;
      const AProjectName: string);
    class procedure PatchFileNames(const AFilePaths: TArray<string>;
      const AProjectName: string);
  public
    class constructor Create;
    class destructor Destroy;
    class procedure NewModule(const ADirectory: string;
      const AModuleType: TModuleType; const AModuleName: string);
    class procedure NewProject(const ADirectory: string;
      const AProjectType: TProjectType; const AProjectName: string);
  end;

implementation

uses
  System.IOUtils,
  System.Zip, DelphiCli.ResourceStrings;

class constructor TTemplateManager.Create;
begin
  inherited;
  FTemplateNames := TDictionary<TProjectType, string>.Create;
  FTemplateNames.Add(TProjectType.Con, 'TemplateCon');
  FTemplateNames.Add(TProjectType.Dpk, 'TemplateDpk');
  FTemplateNames.Add(TProjectType.Vcl, 'TemplateVcl');
  FTemplateNames.Add(TProjectType.Fmx, 'TemplateFmx');
end;

class destructor TTemplateManager.Destroy;
begin
  inherited;
  FreeAndNil(FTemplateNames);
end;

class procedure TTemplateManager.ExtractTemplateToDirectory(const ATemplatName
  : string; const ADestPath: string);
var
  LPath: string;
  LFileStream: TFileStream;
  LStream: TResourceStream;
begin

  // Ottiene il percorso del file temporaneo.
  LPath := TPath.GetTempFileName;

  try

    // Legge la risorsa che contiene il template e scrive i dati nel file.
    LFileStream := TFileStream.Create(LPath, fmCreate);
    try
      LStream := TResourceStream.Create(HInstance, ATemplatName, PChar('ZIP'));
      try
        LStream.SaveToStream(LFileStream);
      finally
        LStream.Free;
      end;
    finally
      LFileStream.Free;
    end;

    // Estrae il contenuto del file ZIP nel percorso indicato.
    TZipFile.ExtractZipFile(LPath, ADestPath, nil);

  finally

    // Garantisce l'eliminazione del file temporaneo.
    TFile.Delete(LPath);

  end;

end;

class procedure TTemplateManager.NewModule(const ADirectory: string;
  const AModuleType: TModuleType; const AModuleName: string);
var
  LFilePath: string;
  LWriter: TStreamWriter;
begin
  // Forza la creazione del percorso di destinazione.
  TDirectory.CreateDirectory(ADirectory);

  // Crea il file (o i file) che costituiscono il modulo.
  case AModuleType of
    &Unit:
      begin
        LFilePath := TPath.Combine(ADirectory, AModuleName + '.pas');
        LWriter := TStreamWriter.Create(LFilePath);
        try
          LWriter.WriteLine(Format('unit %s;', [AModuleName]));
          LWriter.WriteLine();
          LWriter.WriteLine('interface');
          LWriter.WriteLine();
          LWriter.WriteLine('implementation');
          LWriter.WriteLine();
          LWriter.WriteLine('end.');
        finally
          LWriter.Free;
        end;
      end;
    Form:
      begin
        // TODO
      end;
    Frame:
      begin
        // TODO
      end;
    DataModule:
      begin
        // TODO
      end;
  end;
end;

class procedure TTemplateManager.NewProject(const ADirectory: string;
  const AProjectType: TProjectType; const AProjectName: string);
var
  LTemplateName: string;
  LTemplateFiles: TArray<string>;
begin

  // Recupera il nome del template associato al tipo di progetto.
  if not FTemplateNames.TryGetValue(AProjectType, LTemplateName) then
    raise Exception.Create(StrErrProjectTypeUnsupported);

  // Forza la creazione del percorso di destinazione.
  TDirectory.CreateDirectory(ADirectory);

  // Estrae i file del template nel percorso indicato.
  ExtractTemplateToDirectory(LTemplateName, ADirectory);

  // Recupera la lista dei file da sottoporre a patch.
  LTemplateFiles := TDirectory.GetFiles(ADirectory, 'Template.*',
    TSearchOption.soAllDirectories);

  // Esegue la patch del contenuto dei file del template.
  PatchFileContents(LTemplateFiles, AProjectName);

  // Esegue la patch dei nomi dei file del template.
  PatchFileNames(LTemplateFiles, AProjectName);

end;

class procedure TTemplateManager.PatchFileContents(const AFilePaths
  : TArray<string>; const AProjectName: string);
var
  LFilePath: string;
  LLineText: string;
begin
  for LFilePath in AFilePaths do
  begin
    LLineText := TFile.ReadAllText(LFilePath);
    LLineText := StringReplace(LLineText, 'Template', AProjectName,
      [rfReplaceAll, rfIgnoreCase]);
    TFile.WriteAllText(LFilePath, LLineText);
  end;
end;

class procedure TTemplateManager.PatchFileNames(const AFilePaths
  : TArray<string>; const AProjectName: string);
var
  LDirPath: string;
  LFilePath: string;
  LFileName: string;
begin
  for LFilePath in AFilePaths do
  begin
    LDirPath := TPath.GetDirectoryName(LFilePath);
    LFileName := StringReplace(TPath.GetFileName(LFilePath), 'Template',
      AProjectName, [rfReplaceAll, rfIgnoreCase]);
    TFile.Move(LFilePath, TPath.Combine(LDirPath, LFileName));
  end;
end;

end.
